package net.lemnik.eodsql.spi.util;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Map;
import java.util.Set;
import net.lemnik.eodsql.InvalidQueryException;
import net.lemnik.eodsql.spi.util.DataObjectBinding.BindingType;

/**
 * This class simply adds the methods to deal with
 * a DataObjectBinding to ResultSetWrapper.
 * 
 * Created on 2008/07/21
 * @author Jason Morris
 */
abstract class AbstractResultSetWrapper<T, V> extends ResultSetWrapper<T> {
    protected DataObjectBinding<V> binding;

    protected AbstractResultSetWrapper(DataObjectBinding<V> binding) {
        this.binding = binding;
    }

    protected DataObjectBinding<V> getDataObjectBinding() {
        return binding;
    }

    @Override
    public String[] getKeyColumnNames() {
        return binding.getKeyColumnNames();
    }

    static Class<?> getDataObjectClass(Type genericType, Set<Class> outerTypes) {
        if(genericType instanceof ParameterizedType) {
            ParameterizedType paramType = (ParameterizedType)genericType;

            if(outerTypes.contains(paramType.getRawType())) {
                Type[] args = paramType.getActualTypeArguments();

                if(args.length != 1 || !(args[0] instanceof Class)) {
                    throw new InvalidQueryException("Generic must have a single solid type-argument");
                }

                Class<?> clazz = (Class<?>)args[0];
                DataObjectBinding.validate(clazz);
                
                return clazz;
            }
        }

        return null;
    }

}
