package net.lemnik.eodsql.spi.util;

import java.lang.reflect.InvocationTargetException;

/**
 *
 * @author jason
 */
interface MethodParameterTool {
    public Object getParameter(Object... params);
    public Class getParameterType();
    
    // <editor-fold defaultstate="collapsed" desc=" Default ">
    
    class Parameter implements MethodParameterTool {
        private int paramIndex;
        private Class type;
        
        /** Creates a new instance of DefaultParameterHandler */
        public Parameter(int paramIndex, Class type) {
            this.paramIndex = paramIndex;
            this.type = type;
        }
        
        public Object getParameter(Object... params) {
            return params[paramIndex];
        }
        
        public Class getParameterType() {
            return type;
        }
        
    }
    
    // </editor-fold>
    
    // <editor-fold defaultstate="collapsed" desc=" Field ">
    
    class Field implements MethodParameterTool {
        private int paramIndex;
        private java.lang.reflect.Field field;
        
        /** Creates a new instance of FieldParameterHandler */
        public Field(int paramIndex, java.lang.reflect.Field field) {
            this.paramIndex = paramIndex;
            this.field = field;
        }
        
        public Object getParameter(Object... params) {
            try {
                return field.get(params[paramIndex]);
            } catch(IllegalArgumentException ex) {
                throw new RuntimeException(ex);
            } catch(IllegalAccessException ex) {
                throw new RuntimeException(ex);
            }
        }
        
        public Class getParameterType() {
            return field.getType();
        }
        
    }
    
    // </editor-fold>
    
    // <editor-fold defaultstate="collapsed" desc=" Method ">
    
    class Method implements MethodParameterTool {
        private int paramIndex;
        private java.lang.reflect.Method method;
        
        /** Creates a new instance of MethodParameterHandler */
        public Method(int paramIndex, java.lang.reflect.Method method) {
            this.paramIndex = paramIndex;
            this.method = method;
        }
        
        public Object getParameter(Object... params) {
            try {
                return method.invoke(params[paramIndex]);
            } catch(IllegalArgumentException ex) {
                throw new RuntimeException(ex);
            } catch(IllegalAccessException ex) {
                throw new RuntimeException(ex);
            } catch(InvocationTargetException ex) {
                throw new RuntimeException(ex);
            }
        }
        
        public Class getParameterType() {
            return method.getReturnType();
        }
        
    }
    
    // </editor-fold>
    
    // <editor-fold defaultstate="collapsed" desc=" Chained ">
    
    class Chained implements MethodParameterTool {
        private MethodParameterTool from;
        private MethodParameterTool to;
        
        /** Creates a new instance of ChainedParameterHandler */
        public Chained(MethodParameterTool from, MethodParameterTool to) {
            this.from = from;
            this.to = to;
        }
        
        public Object getParameter(Object... params) {
            return to.getParameter(from.getParameter(params));
        }
        
        public Class getParameterType() {
            return to.getParameterType();
        }
        
    }
    
    // </editor-fold>
}
