package net.lemnik.eodsql.spi.util;

import net.lemnik.eodsql.spi.Context;
import java.sql.SQLException;

import java.util.Collection;

import net.lemnik.eodsql.DataSetCache;

/**
 *
 * @author jason
 */
class UpdatableDataSet<T> extends ConnectedDataSet<T> {
    UpdatableDataSet(DataObjectBinding<T> binding, Context context, DataSetCache<T> cache) {
        super(binding, context, cache);
    }

    @Override
    public void disconnect() {
        throw new UnsupportedOperationException("Updatable DataSet's cannot be disconnected.");
    }

    @Override
    public boolean add(T e) {
        try {
            results.moveToInsertRow();
            binding.marshall(e, results);
            results.insertRow();
            
            setSize(null);
        } catch(SQLException ex) {
            throw new RuntimeException(ex);
        }
        
        return true;
    }

    @Override
    public boolean addAll(Collection<? extends T> c) {
        if(c.isEmpty()) {
            return false;
        } else {
            for(T o : c) {
                add(o);
            }
            
            return true;
        }
    }

    @Override
    public T remove(int index) {
        T value = get(index);
        
        try {
            results.absolute(index + 1);
            results.deleteRow();
            setSize(null);
        } catch(SQLException ex) {
            throw new RuntimeException(ex);
        }
        
        return value;
    }

    @Override
    public T set(int index, T element) {
        T oldValue = get(index);
        
        try {
            results.absolute(index + 1);
            binding.marshall(element, results);
            results.updateRow();
        } catch(SQLException ex) {
            throw new RuntimeException(ex);
        }
        
        return oldValue;
    }
    
}
